# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import money_pb2  # type: ignore
from google.type import postal_address_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.domains.v1",
    manifest={
        "ContactPrivacy",
        "DomainNotice",
        "ContactNotice",
        "TransferLockState",
        "Registration",
        "ManagementSettings",
        "DnsSettings",
        "ContactSettings",
        "SearchDomainsRequest",
        "SearchDomainsResponse",
        "RetrieveRegisterParametersRequest",
        "RetrieveRegisterParametersResponse",
        "RegisterDomainRequest",
        "RetrieveTransferParametersRequest",
        "RetrieveTransferParametersResponse",
        "TransferDomainRequest",
        "ListRegistrationsRequest",
        "ListRegistrationsResponse",
        "GetRegistrationRequest",
        "UpdateRegistrationRequest",
        "ConfigureManagementSettingsRequest",
        "ConfigureDnsSettingsRequest",
        "ConfigureContactSettingsRequest",
        "ExportRegistrationRequest",
        "DeleteRegistrationRequest",
        "RetrieveAuthorizationCodeRequest",
        "ResetAuthorizationCodeRequest",
        "RegisterParameters",
        "TransferParameters",
        "AuthorizationCode",
        "OperationMetadata",
    },
)


class ContactPrivacy(proto.Enum):
    r"""Defines a set of possible contact privacy settings for a
    ``Registration``.

    `ICANN <https://icann.org/>`__ maintains the WHOIS database, a
    publicly accessible mapping from domain name to contact information,
    and requires that each domain name have an entry. Choose from these
    options to control how much information in your ``ContactSettings``
    is published.

    Values:
        CONTACT_PRIVACY_UNSPECIFIED (0):
            The contact privacy settings are undefined.
        PUBLIC_CONTACT_DATA (1):
            All the data from ``ContactSettings`` is publicly available.
            When setting this option, you must also provide a
            ``PUBLIC_CONTACT_DATA_ACKNOWLEDGEMENT`` in the
            ``contact_notices`` field of the request.
        PRIVATE_CONTACT_DATA (2):
            None of the data from ``ContactSettings`` is publicly
            available. Instead, proxy contact data is published for your
            domain. Email sent to the proxy email address is forwarded
            to the registrant's email address. Cloud Domains provides
            this privacy proxy service at no additional cost.
        REDACTED_CONTACT_DATA (3):
            Some data from ``ContactSettings`` is publicly available.
            The actual information redacted depends on the domain. For
            details, see `the registration privacy
            article <https://support.google.com/domains/answer/3251242>`__.
    """
    CONTACT_PRIVACY_UNSPECIFIED = 0
    PUBLIC_CONTACT_DATA = 1
    PRIVATE_CONTACT_DATA = 2
    REDACTED_CONTACT_DATA = 3


class DomainNotice(proto.Enum):
    r"""Notices about special properties of certain domains.

    Values:
        DOMAIN_NOTICE_UNSPECIFIED (0):
            The notice is undefined.
        HSTS_PRELOADED (1):
            Indicates that the domain is preloaded on the HTTP Strict
            Transport Security list in browsers. Serving a website on
            such domain requires an SSL certificate. For details, see
            `how to get an SSL
            certificate <https://support.google.com/domains/answer/7638036>`__.
    """
    DOMAIN_NOTICE_UNSPECIFIED = 0
    HSTS_PRELOADED = 1


class ContactNotice(proto.Enum):
    r"""Notices related to contact information.

    Values:
        CONTACT_NOTICE_UNSPECIFIED (0):
            The notice is undefined.
        PUBLIC_CONTACT_DATA_ACKNOWLEDGEMENT (1):
            Required when setting the ``privacy`` field of
            ``ContactSettings`` to ``PUBLIC_CONTACT_DATA``, which
            exposes contact data publicly.
    """
    CONTACT_NOTICE_UNSPECIFIED = 0
    PUBLIC_CONTACT_DATA_ACKNOWLEDGEMENT = 1


class TransferLockState(proto.Enum):
    r"""Possible states of a ``Registration``'s transfer lock.

    Values:
        TRANSFER_LOCK_STATE_UNSPECIFIED (0):
            The state is unspecified.
        UNLOCKED (1):
            The domain is unlocked and can be transferred
            to another registrar.
        LOCKED (2):
            The domain is locked and cannot be
            transferred to another registrar.
    """
    TRANSFER_LOCK_STATE_UNSPECIFIED = 0
    UNLOCKED = 1
    LOCKED = 2


class Registration(proto.Message):
    r"""The ``Registration`` resource facilitates managing and configuring
    domain name registrations.

    There are several ways to create a new ``Registration`` resource:

    To create a new ``Registration`` resource, find a suitable domain
    name by calling the ``SearchDomains`` method with a query to see
    available domain name options. After choosing a name, call
    ``RetrieveRegisterParameters`` to ensure availability and obtain
    information like pricing, which is needed to build a call to
    ``RegisterDomain``.

    Another way to create a new ``Registration`` is to transfer an
    existing domain from another registrar. First, go to the current
    registrar to unlock the domain for transfer and retrieve the
    domain's transfer authorization code. Then call
    ``RetrieveTransferParameters`` to confirm that the domain is
    unlocked and to get values needed to build a call to
    ``TransferDomain``.

    Attributes:
        name (str):
            Output only. Name of the ``Registration`` resource, in the
            format
            ``projects/*/locations/*/registrations/<domain_name>``.
        domain_name (str):
            Required. Immutable. The domain name. Unicode
            domain names must be expressed in Punycode
            format.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The creation timestamp of the ``Registration``
            resource.
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The expiration timestamp of the
            ``Registration``.
        state (google.cloud.domains_v1.types.Registration.State):
            Output only. The state of the ``Registration``
        issues (MutableSequence[google.cloud.domains_v1.types.Registration.Issue]):
            Output only. The set of issues with the ``Registration``
            that require attention.
        labels (MutableMapping[str, str]):
            Set of labels associated with the ``Registration``.
        management_settings (google.cloud.domains_v1.types.ManagementSettings):
            Settings for management of the ``Registration``, including
            renewal, billing, and transfer. You cannot update these with
            the ``UpdateRegistration`` method. To update these settings,
            use the ``ConfigureManagementSettings`` method.
        dns_settings (google.cloud.domains_v1.types.DnsSettings):
            Settings controlling the DNS configuration of the
            ``Registration``. You cannot update these with the
            ``UpdateRegistration`` method. To update these settings, use
            the ``ConfigureDnsSettings`` method.
        contact_settings (google.cloud.domains_v1.types.ContactSettings):
            Required. Settings for contact information linked to the
            ``Registration``. You cannot update these with the
            ``UpdateRegistration`` method. To update these settings, use
            the ``ConfigureContactSettings`` method.
        pending_contact_settings (google.cloud.domains_v1.types.ContactSettings):
            Output only. Pending contact settings for the
            ``Registration``. Updates to the ``contact_settings`` field
            that change its ``registrant_contact`` or ``privacy`` fields
            require email confirmation by the ``registrant_contact``
            before taking effect. This field is set only if there are
            pending updates to the ``contact_settings`` that have not
            been confirmed. To confirm the changes, the
            ``registrant_contact`` must follow the instructions in the
            email they receive.
        supported_privacy (MutableSequence[google.cloud.domains_v1.types.ContactPrivacy]):
            Output only. Set of options for the
            ``contact_settings.privacy`` field that this
            ``Registration`` supports.
    """

    class State(proto.Enum):
        r"""Possible states of a ``Registration``.

        Values:
            STATE_UNSPECIFIED (0):
                The state is undefined.
            REGISTRATION_PENDING (1):
                The domain is being registered.
            REGISTRATION_FAILED (2):
                The domain registration failed. You can
                delete resources in this state to allow
                registration to be retried.
            TRANSFER_PENDING (3):
                The domain is being transferred from another
                registrar to Cloud Domains.
            TRANSFER_FAILED (4):
                The attempt to transfer the domain from
                another registrar to Cloud Domains failed. You
                can delete resources in this state and retry the
                transfer.
            ACTIVE (6):
                The domain is registered and operational. The
                domain renews automatically as long as it
                remains in this state.
            SUSPENDED (7):
                The domain is suspended and inoperative. For more details,
                see the ``issues`` field.
            EXPORTED (8):
                The domain is no longer managed with Cloud Domains. It may
                have been transferred to another registrar or exported for
                management in `Google Domains <https://domains.google/>`__.
                You can no longer update it with this API, and information
                shown about it may be stale. Domains in this state are not
                automatically renewed by Cloud Domains.
        """
        STATE_UNSPECIFIED = 0
        REGISTRATION_PENDING = 1
        REGISTRATION_FAILED = 2
        TRANSFER_PENDING = 3
        TRANSFER_FAILED = 4
        ACTIVE = 6
        SUSPENDED = 7
        EXPORTED = 8

    class Issue(proto.Enum):
        r"""Possible issues with a ``Registration`` that require attention.

        Values:
            ISSUE_UNSPECIFIED (0):
                The issue is undefined.
            CONTACT_SUPPORT (1):
                Contact the Cloud Support team to resolve a
                problem with this domain.
            UNVERIFIED_EMAIL (2):
                `ICANN <https://icann.org/>`__ requires verification of the
                email address in the ``Registration``'s
                ``contact_settings.registrant_contact`` field. To verify the
                email address, follow the instructions in the email the
                ``registrant_contact`` receives following registration. If
                you do not complete email verification within 15 days of
                registration, the domain is suspended. To resend the
                verification email, call ConfigureContactSettings and
                provide the current ``registrant_contact.email``.
        """
        ISSUE_UNSPECIFIED = 0
        CONTACT_SUPPORT = 1
        UNVERIFIED_EMAIL = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    domain_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    expire_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )
    issues: MutableSequence[Issue] = proto.RepeatedField(
        proto.ENUM,
        number=8,
        enum=Issue,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=9,
    )
    management_settings: "ManagementSettings" = proto.Field(
        proto.MESSAGE,
        number=10,
        message="ManagementSettings",
    )
    dns_settings: "DnsSettings" = proto.Field(
        proto.MESSAGE,
        number=11,
        message="DnsSettings",
    )
    contact_settings: "ContactSettings" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="ContactSettings",
    )
    pending_contact_settings: "ContactSettings" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="ContactSettings",
    )
    supported_privacy: MutableSequence["ContactPrivacy"] = proto.RepeatedField(
        proto.ENUM,
        number=14,
        enum="ContactPrivacy",
    )


class ManagementSettings(proto.Message):
    r"""Defines renewal, billing, and transfer settings for a
    ``Registration``.

    Attributes:
        renewal_method (google.cloud.domains_v1.types.ManagementSettings.RenewalMethod):
            Output only. The renewal method for this ``Registration``.
        transfer_lock_state (google.cloud.domains_v1.types.TransferLockState):
            Controls whether the domain can be
            transferred to another registrar.
    """

    class RenewalMethod(proto.Enum):
        r"""Defines how the ``Registration`` is renewed.

        Values:
            RENEWAL_METHOD_UNSPECIFIED (0):
                The renewal method is undefined.
            AUTOMATIC_RENEWAL (1):
                The domain is automatically renewed each year .

                To disable automatic renewals, delete the resource by
                calling ``DeleteRegistration`` or export it by calling
                ``ExportRegistration``.
            MANUAL_RENEWAL (2):
                The domain must be explicitly renewed each year before its
                ``expire_time``. This option is only available when the
                ``Registration`` is in state ``EXPORTED``.

                To manage the domain's current billing and renewal settings,
                go to `Google Domains <https://domains.google/>`__.
        """
        RENEWAL_METHOD_UNSPECIFIED = 0
        AUTOMATIC_RENEWAL = 1
        MANUAL_RENEWAL = 2

    renewal_method: RenewalMethod = proto.Field(
        proto.ENUM,
        number=3,
        enum=RenewalMethod,
    )
    transfer_lock_state: "TransferLockState" = proto.Field(
        proto.ENUM,
        number=4,
        enum="TransferLockState",
    )


class DnsSettings(proto.Message):
    r"""Defines the DNS configuration of a ``Registration``, including name
    servers, DNSSEC, and glue records.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        custom_dns (google.cloud.domains_v1.types.DnsSettings.CustomDns):
            An arbitrary DNS provider identified by its
            name servers.

            This field is a member of `oneof`_ ``dns_provider``.
        google_domains_dns (google.cloud.domains_v1.types.DnsSettings.GoogleDomainsDns):
            The free DNS zone provided by `Google
            Domains <https://domains.google/>`__.

            This field is a member of `oneof`_ ``dns_provider``.
        glue_records (MutableSequence[google.cloud.domains_v1.types.DnsSettings.GlueRecord]):
            The list of glue records for this ``Registration``. Commonly
            empty.
    """

    class DsState(proto.Enum):
        r"""The publication state of DS records for a ``Registration``.

        Values:
            DS_STATE_UNSPECIFIED (0):
                DS state is unspecified.
            DS_RECORDS_UNPUBLISHED (1):
                DNSSEC is disabled for this domain. No DS
                records for this domain are published in the
                parent DNS zone.
            DS_RECORDS_PUBLISHED (2):
                DNSSEC is enabled for this domain. Appropriate DS records
                for this domain are published in the parent DNS zone. This
                option is valid only if the DNS zone referenced in the
                ``Registration``'s ``dns_provider`` field is already
                DNSSEC-signed.
        """
        DS_STATE_UNSPECIFIED = 0
        DS_RECORDS_UNPUBLISHED = 1
        DS_RECORDS_PUBLISHED = 2

    class CustomDns(proto.Message):
        r"""Configuration for an arbitrary DNS provider.

        Attributes:
            name_servers (MutableSequence[str]):
                Required. A list of name servers that store
                the DNS zone for this domain. Each name server
                is a domain name, with Unicode domain names
                expressed in Punycode format.
            ds_records (MutableSequence[google.cloud.domains_v1.types.DnsSettings.DsRecord]):
                The list of DS records for this domain, which
                are used to enable DNSSEC. The domain's DNS
                provider can provide the values to set here. If
                this field is empty, DNSSEC is disabled.
        """

        name_servers: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )
        ds_records: MutableSequence["DnsSettings.DsRecord"] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="DnsSettings.DsRecord",
        )

    class GoogleDomainsDns(proto.Message):
        r"""Configuration for using the free DNS zone provided by Google Domains
        as a ``Registration``'s ``dns_provider``. You cannot configure the
        DNS zone itself using the API. To configure the DNS zone, go to
        `Google Domains <https://domains.google/>`__.

        Attributes:
            name_servers (MutableSequence[str]):
                Output only. A list of name servers that
                store the DNS zone for this domain. Each name
                server is a domain name, with Unicode domain
                names expressed in Punycode format. This field
                is automatically populated with the name servers
                assigned to the Google Domains DNS zone.
            ds_state (google.cloud.domains_v1.types.DnsSettings.DsState):
                Required. The state of DS records for this
                domain. Used to enable or disable automatic
                DNSSEC.
            ds_records (MutableSequence[google.cloud.domains_v1.types.DnsSettings.DsRecord]):
                Output only. The list of DS records published for this
                domain. The list is automatically populated when
                ``ds_state`` is ``DS_RECORDS_PUBLISHED``, otherwise it
                remains empty.
        """

        name_servers: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )
        ds_state: "DnsSettings.DsState" = proto.Field(
            proto.ENUM,
            number=2,
            enum="DnsSettings.DsState",
        )
        ds_records: MutableSequence["DnsSettings.DsRecord"] = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="DnsSettings.DsRecord",
        )

    class DsRecord(proto.Message):
        r"""Defines a Delegation Signer (DS) record, which is needed to
        enable DNSSEC for a domain. It contains a digest (hash) of a
        DNSKEY record that must be present in the domain's DNS zone.

        Attributes:
            key_tag (int):
                The key tag of the record. Must be set in
                range 0 -- 65535.
            algorithm (google.cloud.domains_v1.types.DnsSettings.DsRecord.Algorithm):
                The algorithm used to generate the referenced
                DNSKEY.
            digest_type (google.cloud.domains_v1.types.DnsSettings.DsRecord.DigestType):
                The hash function used to generate the digest
                of the referenced DNSKEY.
            digest (str):
                The digest generated from the referenced
                DNSKEY.
        """

        class Algorithm(proto.Enum):
            r"""List of algorithms used to create a DNSKEY. Certain
            algorithms are not supported for particular domains.

            Values:
                ALGORITHM_UNSPECIFIED (0):
                    The algorithm is unspecified.
                RSAMD5 (1):
                    RSA/MD5. Cannot be used for new deployments.
                DH (2):
                    Diffie-Hellman. Cannot be used for new
                    deployments.
                DSA (3):
                    DSA/SHA1. Not recommended for new
                    deployments.
                ECC (4):
                    ECC. Not recommended for new deployments.
                RSASHA1 (5):
                    RSA/SHA-1. Not recommended for new
                    deployments.
                DSANSEC3SHA1 (6):
                    DSA-NSEC3-SHA1. Not recommended for new
                    deployments.
                RSASHA1NSEC3SHA1 (7):
                    RSA/SHA1-NSEC3-SHA1. Not recommended for new
                    deployments.
                RSASHA256 (8):
                    RSA/SHA-256.
                RSASHA512 (10):
                    RSA/SHA-512.
                ECCGOST (12):
                    GOST R 34.10-2001.
                ECDSAP256SHA256 (13):
                    ECDSA Curve P-256 with SHA-256.
                ECDSAP384SHA384 (14):
                    ECDSA Curve P-384 with SHA-384.
                ED25519 (15):
                    Ed25519.
                ED448 (16):
                    Ed448.
                INDIRECT (252):
                    Reserved for Indirect Keys. Cannot be used
                    for new deployments.
                PRIVATEDNS (253):
                    Private algorithm. Cannot be used for new
                    deployments.
                PRIVATEOID (254):
                    Private algorithm OID. Cannot be used for new
                    deployments.
            """
            ALGORITHM_UNSPECIFIED = 0
            RSAMD5 = 1
            DH = 2
            DSA = 3
            ECC = 4
            RSASHA1 = 5
            DSANSEC3SHA1 = 6
            RSASHA1NSEC3SHA1 = 7
            RSASHA256 = 8
            RSASHA512 = 10
            ECCGOST = 12
            ECDSAP256SHA256 = 13
            ECDSAP384SHA384 = 14
            ED25519 = 15
            ED448 = 16
            INDIRECT = 252
            PRIVATEDNS = 253
            PRIVATEOID = 254

        class DigestType(proto.Enum):
            r"""List of hash functions that may have been used to generate a
            digest of a DNSKEY.

            Values:
                DIGEST_TYPE_UNSPECIFIED (0):
                    The DigestType is unspecified.
                SHA1 (1):
                    SHA-1. Not recommended for new deployments.
                SHA256 (2):
                    SHA-256.
                GOST3411 (3):
                    GOST R 34.11-94.
                SHA384 (4):
                    SHA-384.
            """
            DIGEST_TYPE_UNSPECIFIED = 0
            SHA1 = 1
            SHA256 = 2
            GOST3411 = 3
            SHA384 = 4

        key_tag: int = proto.Field(
            proto.INT32,
            number=1,
        )
        algorithm: "DnsSettings.DsRecord.Algorithm" = proto.Field(
            proto.ENUM,
            number=2,
            enum="DnsSettings.DsRecord.Algorithm",
        )
        digest_type: "DnsSettings.DsRecord.DigestType" = proto.Field(
            proto.ENUM,
            number=3,
            enum="DnsSettings.DsRecord.DigestType",
        )
        digest: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class GlueRecord(proto.Message):
        r"""Defines a host on your domain that is a DNS name server for your
        domain and/or other domains. Glue records are a way of making the IP
        address of a name server known, even when it serves DNS queries for
        its parent domain. For example, when ``ns.example.com`` is a name
        server for ``example.com``, the host ``ns.example.com`` must have a
        glue record to break the circular DNS reference.

        Attributes:
            host_name (str):
                Required. Domain name of the host in Punycode
                format.
            ipv4_addresses (MutableSequence[str]):
                List of IPv4 addresses corresponding to this host in the
                standard decimal format (e.g. ``198.51.100.1``). At least
                one of ``ipv4_address`` and ``ipv6_address`` must be set.
            ipv6_addresses (MutableSequence[str]):
                List of IPv6 addresses corresponding to this host in the
                standard hexadecimal format (e.g. ``2001:db8::``). At least
                one of ``ipv4_address`` and ``ipv6_address`` must be set.
        """

        host_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        ipv4_addresses: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )
        ipv6_addresses: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )

    custom_dns: CustomDns = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="dns_provider",
        message=CustomDns,
    )
    google_domains_dns: GoogleDomainsDns = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="dns_provider",
        message=GoogleDomainsDns,
    )
    glue_records: MutableSequence[GlueRecord] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=GlueRecord,
    )


class ContactSettings(proto.Message):
    r"""Defines the contact information associated with a ``Registration``.

    `ICANN <https://icann.org/>`__ requires all domain names to have
    associated contact information. The ``registrant_contact`` is
    considered the domain's legal owner, and often the other contacts
    are identical.

    Attributes:
        privacy (google.cloud.domains_v1.types.ContactPrivacy):
            Required. Privacy setting for the contacts associated with
            the ``Registration``.
        registrant_contact (google.cloud.domains_v1.types.ContactSettings.Contact):
            Required. The registrant contact for the ``Registration``.

            *Caution: Anyone with access to this email address, phone
            number, and/or postal address can take control of the
            domain.*

            *Warning: For new ``Registration``\ s, the registrant
            receives an email confirmation that they must complete
            within 15 days to avoid domain suspension.*
        admin_contact (google.cloud.domains_v1.types.ContactSettings.Contact):
            Required. The administrative contact for the
            ``Registration``.
        technical_contact (google.cloud.domains_v1.types.ContactSettings.Contact):
            Required. The technical contact for the ``Registration``.
    """

    class Contact(proto.Message):
        r"""Details required for a contact associated with a ``Registration``.

        Attributes:
            postal_address (google.type.postal_address_pb2.PostalAddress):
                Required. Postal address of the contact.
            email (str):
                Required. Email address of the contact.
            phone_number (str):
                Required. Phone number of the contact in international
                format. For example, ``"+1-800-555-0123"``.
            fax_number (str):
                Fax number of the contact in international format. For
                example, ``"+1-800-555-0123"``.
        """

        postal_address: postal_address_pb2.PostalAddress = proto.Field(
            proto.MESSAGE,
            number=1,
            message=postal_address_pb2.PostalAddress,
        )
        email: str = proto.Field(
            proto.STRING,
            number=2,
        )
        phone_number: str = proto.Field(
            proto.STRING,
            number=3,
        )
        fax_number: str = proto.Field(
            proto.STRING,
            number=4,
        )

    privacy: "ContactPrivacy" = proto.Field(
        proto.ENUM,
        number=1,
        enum="ContactPrivacy",
    )
    registrant_contact: Contact = proto.Field(
        proto.MESSAGE,
        number=2,
        message=Contact,
    )
    admin_contact: Contact = proto.Field(
        proto.MESSAGE,
        number=3,
        message=Contact,
    )
    technical_contact: Contact = proto.Field(
        proto.MESSAGE,
        number=4,
        message=Contact,
    )


class SearchDomainsRequest(proto.Message):
    r"""Request for the ``SearchDomains`` method.

    Attributes:
        query (str):
            Required. String used to search for available
            domain names.
        location (str):
            Required. The location. Must be in the format
            ``projects/*/locations/*``.
    """

    query: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SearchDomainsResponse(proto.Message):
    r"""Response for the ``SearchDomains`` method.

    Attributes:
        register_parameters (MutableSequence[google.cloud.domains_v1.types.RegisterParameters]):
            Results of the domain name search.
    """

    register_parameters: MutableSequence["RegisterParameters"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="RegisterParameters",
    )


class RetrieveRegisterParametersRequest(proto.Message):
    r"""Request for the ``RetrieveRegisterParameters`` method.

    Attributes:
        domain_name (str):
            Required. The domain name. Unicode domain
            names must be expressed in Punycode format.
        location (str):
            Required. The location. Must be in the format
            ``projects/*/locations/*``.
    """

    domain_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )


class RetrieveRegisterParametersResponse(proto.Message):
    r"""Response for the ``RetrieveRegisterParameters`` method.

    Attributes:
        register_parameters (google.cloud.domains_v1.types.RegisterParameters):
            Parameters to use when calling the ``RegisterDomain``
            method.
    """

    register_parameters: "RegisterParameters" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="RegisterParameters",
    )


class RegisterDomainRequest(proto.Message):
    r"""Request for the ``RegisterDomain`` method.

    Attributes:
        parent (str):
            Required. The parent resource of the ``Registration``. Must
            be in the format ``projects/*/locations/*``.
        registration (google.cloud.domains_v1.types.Registration):
            Required. The complete ``Registration`` resource to be
            created.
        domain_notices (MutableSequence[google.cloud.domains_v1.types.DomainNotice]):
            The list of domain notices that you acknowledge. Call
            ``RetrieveRegisterParameters`` to see the notices that need
            acknowledgement.
        contact_notices (MutableSequence[google.cloud.domains_v1.types.ContactNotice]):
            The list of contact notices that the caller acknowledges.
            The notices needed here depend on the values specified in
            ``registration.contact_settings``.
        yearly_price (google.type.money_pb2.Money):
            Required. Yearly price to register or renew
            the domain. The value that should be put here
            can be obtained from RetrieveRegisterParameters
            or SearchDomains calls.
        validate_only (bool):
            When true, only validation is performed, without actually
            registering the domain. Follows:
            https://cloud.google.com/apis/design/design_patterns#request_validation
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    registration: "Registration" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="Registration",
    )
    domain_notices: MutableSequence["DomainNotice"] = proto.RepeatedField(
        proto.ENUM,
        number=3,
        enum="DomainNotice",
    )
    contact_notices: MutableSequence["ContactNotice"] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum="ContactNotice",
    )
    yearly_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=5,
        message=money_pb2.Money,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class RetrieveTransferParametersRequest(proto.Message):
    r"""Request for the ``RetrieveTransferParameters`` method.

    Attributes:
        domain_name (str):
            Required. The domain name. Unicode domain
            names must be expressed in Punycode format.
        location (str):
            Required. The location. Must be in the format
            ``projects/*/locations/*``.
    """

    domain_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )


class RetrieveTransferParametersResponse(proto.Message):
    r"""Response for the ``RetrieveTransferParameters`` method.

    Attributes:
        transfer_parameters (google.cloud.domains_v1.types.TransferParameters):
            Parameters to use when calling the ``TransferDomain``
            method.
    """

    transfer_parameters: "TransferParameters" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="TransferParameters",
    )


class TransferDomainRequest(proto.Message):
    r"""Request for the ``TransferDomain`` method.

    Attributes:
        parent (str):
            Required. The parent resource of the ``Registration``. Must
            be in the format ``projects/*/locations/*``.
        registration (google.cloud.domains_v1.types.Registration):
            Required. The complete ``Registration`` resource to be
            created.

            You can leave ``registration.dns_settings`` unset to import
            the domain's current DNS configuration from its current
            registrar. Use this option only if you are sure that the
            domain's current DNS service does not cease upon transfer,
            as is often the case for DNS services provided for free by
            the registrar.
        contact_notices (MutableSequence[google.cloud.domains_v1.types.ContactNotice]):
            The list of contact notices that you acknowledge. The
            notices needed here depend on the values specified in
            ``registration.contact_settings``.
        yearly_price (google.type.money_pb2.Money):
            Required. Acknowledgement of the price to transfer or renew
            the domain for one year. Call ``RetrieveTransferParameters``
            to obtain the price, which you must acknowledge.
        authorization_code (google.cloud.domains_v1.types.AuthorizationCode):
            The domain's transfer authorization code. You
            can obtain this from the domain's current
            registrar.
        validate_only (bool):
            Validate the request without actually
            transferring the domain.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    registration: "Registration" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="Registration",
    )
    contact_notices: MutableSequence["ContactNotice"] = proto.RepeatedField(
        proto.ENUM,
        number=3,
        enum="ContactNotice",
    )
    yearly_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=4,
        message=money_pb2.Money,
    )
    authorization_code: "AuthorizationCode" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="AuthorizationCode",
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class ListRegistrationsRequest(proto.Message):
    r"""Request for the ``ListRegistrations`` method.

    Attributes:
        parent (str):
            Required. The project and location from which to list
            ``Registration``\ s, specified in the format
            ``projects/*/locations/*``.
        page_size (int):
            Maximum number of results to return.
        page_token (str):
            When set to the ``next_page_token`` from a prior response,
            provides the next page of results.
        filter (str):
            Filter expression to restrict the ``Registration``\ s
            returned.

            The expression must specify the field name, a comparison
            operator, and the value that you want to use for filtering.
            The value must be a string, a number, a boolean, or an enum
            value. The comparison operator should be one of =, !=, >, <,
            >=, <=, or : for prefix or wildcard matches.

            For example, to filter to a specific domain name, use an
            expression like ``domainName="example.com"``. You can also
            check for the existence of a field; for example, to find
            domains using custom DNS settings, use an expression like
            ``dnsSettings.customDns:*``.

            You can also create compound filters by combining
            expressions with the ``AND`` and ``OR`` operators. For
            example, to find domains that are suspended or have specific
            issues flagged, use an expression like
            ``(state=SUSPENDED) OR (issue:*)``.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListRegistrationsResponse(proto.Message):
    r"""Response for the ``ListRegistrations`` method.

    Attributes:
        registrations (MutableSequence[google.cloud.domains_v1.types.Registration]):
            A list of ``Registration``\ s.
        next_page_token (str):
            When present, there are more results to retrieve. Set
            ``page_token`` to this value on a subsequent call to get the
            next page of results.
    """

    @property
    def raw_page(self):
        return self

    registrations: MutableSequence["Registration"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Registration",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetRegistrationRequest(proto.Message):
    r"""Request for the ``GetRegistration`` method.

    Attributes:
        name (str):
            Required. The name of the ``Registration`` to get, in the
            format ``projects/*/locations/*/registrations/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdateRegistrationRequest(proto.Message):
    r"""Request for the ``UpdateRegistration`` method.

    Attributes:
        registration (google.cloud.domains_v1.types.Registration):
            Fields of the ``Registration`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The field mask describing which fields to update
            as a comma-separated list. For example, if only the labels
            are being updated, the ``update_mask`` is ``"labels"``.
    """

    registration: "Registration" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="Registration",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class ConfigureManagementSettingsRequest(proto.Message):
    r"""Request for the ``ConfigureManagementSettings`` method.

    Attributes:
        registration (str):
            Required. The name of the ``Registration`` whose management
            settings are being updated, in the format
            ``projects/*/locations/*/registrations/*``.
        management_settings (google.cloud.domains_v1.types.ManagementSettings):
            Fields of the ``ManagementSettings`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The field mask describing which fields to update
            as a comma-separated list. For example, if only the transfer
            lock is being updated, the ``update_mask`` is
            ``"transfer_lock_state"``.
    """

    registration: str = proto.Field(
        proto.STRING,
        number=1,
    )
    management_settings: "ManagementSettings" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ManagementSettings",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )


class ConfigureDnsSettingsRequest(proto.Message):
    r"""Request for the ``ConfigureDnsSettings`` method.

    Attributes:
        registration (str):
            Required. The name of the ``Registration`` whose DNS
            settings are being updated, in the format
            ``projects/*/locations/*/registrations/*``.
        dns_settings (google.cloud.domains_v1.types.DnsSettings):
            Fields of the ``DnsSettings`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The field mask describing which fields to update
            as a comma-separated list. For example, if only the name
            servers are being updated for an existing Custom DNS
            configuration, the ``update_mask`` is
            ``"custom_dns.name_servers"``.

            When changing the DNS provider from one type to another,
            pass the new provider's field name as part of the field
            mask. For example, when changing from a Google Domains DNS
            configuration to a Custom DNS configuration, the
            ``update_mask`` is ``"custom_dns"``. //
        validate_only (bool):
            Validate the request without actually
            updating the DNS settings.
    """

    registration: str = proto.Field(
        proto.STRING,
        number=1,
    )
    dns_settings: "DnsSettings" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="DnsSettings",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class ConfigureContactSettingsRequest(proto.Message):
    r"""Request for the ``ConfigureContactSettings`` method.

    Attributes:
        registration (str):
            Required. The name of the ``Registration`` whose contact
            settings are being updated, in the format
            ``projects/*/locations/*/registrations/*``.
        contact_settings (google.cloud.domains_v1.types.ContactSettings):
            Fields of the ``ContactSettings`` to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The field mask describing which fields to update
            as a comma-separated list. For example, if only the
            registrant contact is being updated, the ``update_mask`` is
            ``"registrant_contact"``.
        contact_notices (MutableSequence[google.cloud.domains_v1.types.ContactNotice]):
            The list of contact notices that the caller acknowledges.
            The notices needed here depend on the values specified in
            ``contact_settings``.
        validate_only (bool):
            Validate the request without actually
            updating the contact settings.
    """

    registration: str = proto.Field(
        proto.STRING,
        number=1,
    )
    contact_settings: "ContactSettings" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ContactSettings",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )
    contact_notices: MutableSequence["ContactNotice"] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum="ContactNotice",
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


class ExportRegistrationRequest(proto.Message):
    r"""Request for the ``ExportRegistration`` method.

    Attributes:
        name (str):
            Required. The name of the ``Registration`` to export, in the
            format ``projects/*/locations/*/registrations/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DeleteRegistrationRequest(proto.Message):
    r"""Request for the ``DeleteRegistration`` method.

    Attributes:
        name (str):
            Required. The name of the ``Registration`` to delete, in the
            format ``projects/*/locations/*/registrations/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RetrieveAuthorizationCodeRequest(proto.Message):
    r"""Request for the ``RetrieveAuthorizationCode`` method.

    Attributes:
        registration (str):
            Required. The name of the ``Registration`` whose
            authorization code is being retrieved, in the format
            ``projects/*/locations/*/registrations/*``.
    """

    registration: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ResetAuthorizationCodeRequest(proto.Message):
    r"""Request for the ``ResetAuthorizationCode`` method.

    Attributes:
        registration (str):
            Required. The name of the ``Registration`` whose
            authorization code is being reset, in the format
            ``projects/*/locations/*/registrations/*``.
    """

    registration: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RegisterParameters(proto.Message):
    r"""Parameters required to register a new domain.

    Attributes:
        domain_name (str):
            The domain name. Unicode domain names are
            expressed in Punycode format.
        availability (google.cloud.domains_v1.types.RegisterParameters.Availability):
            Indicates whether the domain is available for registration.
            This value is accurate when obtained by calling
            ``RetrieveRegisterParameters``, but is approximate when
            obtained by calling ``SearchDomains``.
        supported_privacy (MutableSequence[google.cloud.domains_v1.types.ContactPrivacy]):
            Contact privacy options that the domain
            supports.
        domain_notices (MutableSequence[google.cloud.domains_v1.types.DomainNotice]):
            Notices about special properties of the
            domain.
        yearly_price (google.type.money_pb2.Money):
            Price to register or renew the domain for one
            year.
    """

    class Availability(proto.Enum):
        r"""Possible availability states of a domain name.

        Values:
            AVAILABILITY_UNSPECIFIED (0):
                The availability is unspecified.
            AVAILABLE (1):
                The domain is available for registration.
            UNAVAILABLE (2):
                The domain is not available for registration.
                Generally this means it is already registered to
                another party.
            UNSUPPORTED (3):
                The domain is not currently supported by
                Cloud Domains, but may be available elsewhere.
            UNKNOWN (4):
                Cloud Domains is unable to determine domain
                availability, generally due to system
                maintenance at the domain name registry.
        """
        AVAILABILITY_UNSPECIFIED = 0
        AVAILABLE = 1
        UNAVAILABLE = 2
        UNSUPPORTED = 3
        UNKNOWN = 4

    domain_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    availability: Availability = proto.Field(
        proto.ENUM,
        number=2,
        enum=Availability,
    )
    supported_privacy: MutableSequence["ContactPrivacy"] = proto.RepeatedField(
        proto.ENUM,
        number=3,
        enum="ContactPrivacy",
    )
    domain_notices: MutableSequence["DomainNotice"] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum="DomainNotice",
    )
    yearly_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=5,
        message=money_pb2.Money,
    )


class TransferParameters(proto.Message):
    r"""Parameters required to transfer a domain from another
    registrar.

    Attributes:
        domain_name (str):
            The domain name. Unicode domain names are
            expressed in Punycode format.
        current_registrar (str):
            The registrar that currently manages the
            domain.
        name_servers (MutableSequence[str]):
            The name servers that currently store the
            configuration of the domain.
        transfer_lock_state (google.cloud.domains_v1.types.TransferLockState):
            Indicates whether the domain is protected by a transfer
            lock. For a transfer to succeed, this must show
            ``UNLOCKED``. To unlock a domain, go to its current
            registrar.
        supported_privacy (MutableSequence[google.cloud.domains_v1.types.ContactPrivacy]):
            Contact privacy options that the domain
            supports.
        yearly_price (google.type.money_pb2.Money):
            Price to transfer or renew the domain for one
            year.
    """

    domain_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    current_registrar: str = proto.Field(
        proto.STRING,
        number=2,
    )
    name_servers: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    transfer_lock_state: "TransferLockState" = proto.Field(
        proto.ENUM,
        number=4,
        enum="TransferLockState",
    )
    supported_privacy: MutableSequence["ContactPrivacy"] = proto.RepeatedField(
        proto.ENUM,
        number=5,
        enum="ContactPrivacy",
    )
    yearly_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=6,
        message=money_pb2.Money,
    )


class AuthorizationCode(proto.Message):
    r"""Defines an authorization code.

    Attributes:
        code (str):
            The Authorization Code in ASCII. It can be
            used to transfer the domain to or from another
            registrar.
    """

    code: str = proto.Field(
        proto.STRING,
        number=1,
    )


class OperationMetadata(proto.Message):
    r"""Represents the metadata of the long-running operation. Output
    only.

    Attributes:
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            The time the operation was created.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            The time the operation finished running.
        target (str):
            Server-defined resource path for the target
            of the operation.
        verb (str):
            Name of the verb executed by the operation.
        status_detail (str):
            Human-readable status of the operation, if
            any.
        api_version (str):
            API version used to start the operation.
    """

    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    target: str = proto.Field(
        proto.STRING,
        number=3,
    )
    verb: str = proto.Field(
        proto.STRING,
        number=4,
    )
    status_detail: str = proto.Field(
        proto.STRING,
        number=5,
    )
    api_version: str = proto.Field(
        proto.STRING,
        number=6,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import logging as std_logging
import re
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf

from google.cloud.domains_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import money_pb2  # type: ignore

from google.cloud.domains_v1.services.domains import pagers
from google.cloud.domains_v1.types import domains

from .client import DomainsClient
from .transports.base import DEFAULT_CLIENT_INFO, DomainsTransport
from .transports.grpc_asyncio import DomainsGrpcAsyncIOTransport

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class DomainsAsyncClient:
    """The Cloud Domains API enables management and configuration of
    domain names.
    """

    _client: DomainsClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = DomainsClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = DomainsClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = DomainsClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = DomainsClient._DEFAULT_UNIVERSE

    registration_path = staticmethod(DomainsClient.registration_path)
    parse_registration_path = staticmethod(DomainsClient.parse_registration_path)
    common_billing_account_path = staticmethod(
        DomainsClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        DomainsClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(DomainsClient.common_folder_path)
    parse_common_folder_path = staticmethod(DomainsClient.parse_common_folder_path)
    common_organization_path = staticmethod(DomainsClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        DomainsClient.parse_common_organization_path
    )
    common_project_path = staticmethod(DomainsClient.common_project_path)
    parse_common_project_path = staticmethod(DomainsClient.parse_common_project_path)
    common_location_path = staticmethod(DomainsClient.common_location_path)
    parse_common_location_path = staticmethod(DomainsClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            DomainsAsyncClient: The constructed client.
        """
        return DomainsClient.from_service_account_info.__func__(DomainsAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            DomainsAsyncClient: The constructed client.
        """
        return DomainsClient.from_service_account_file.__func__(DomainsAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return DomainsClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> DomainsTransport:
        """Returns the transport used by the client instance.

        Returns:
            DomainsTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = DomainsClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, DomainsTransport, Callable[..., DomainsTransport]]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the domains async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,DomainsTransport,Callable[..., DomainsTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the DomainsTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = DomainsClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.cloud.domains_v1.DomainsAsyncClient`.",
                extra={
                    "serviceName": "google.cloud.domains.v1.Domains",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.cloud.domains.v1.Domains",
                    "credentialsType": None,
                },
            )

    async def search_domains(
        self,
        request: Optional[Union[domains.SearchDomainsRequest, dict]] = None,
        *,
        location: Optional[str] = None,
        query: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.SearchDomainsResponse:
        r"""Searches for available domain names similar to the provided
        query.

        Availability results from this method are approximate; call
        ``RetrieveRegisterParameters`` on a domain before registering to
        confirm availability.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_search_domains():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.SearchDomainsRequest(
                    query="query_value",
                    location="location_value",
                )

                # Make the request
                response = await client.search_domains(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.SearchDomainsRequest, dict]]):
                The request object. Request for the ``SearchDomains`` method.
            location (:class:`str`):
                Required. The location. Must be in the format
                ``projects/*/locations/*``.

                This corresponds to the ``location`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            query (:class:`str`):
                Required. String used to search for
                available domain names.

                This corresponds to the ``query`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.SearchDomainsResponse:
                Response for the SearchDomains method.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [location, query]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.SearchDomainsRequest):
            request = domains.SearchDomainsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if location is not None:
            request.location = location
        if query is not None:
            request.query = query

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.search_domains
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("location", request.location),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def retrieve_register_parameters(
        self,
        request: Optional[
            Union[domains.RetrieveRegisterParametersRequest, dict]
        ] = None,
        *,
        location: Optional[str] = None,
        domain_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.RetrieveRegisterParametersResponse:
        r"""Gets parameters needed to register a new domain name, including
        price and up-to-date availability. Use the returned values to
        call ``RegisterDomain``.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_retrieve_register_parameters():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.RetrieveRegisterParametersRequest(
                    domain_name="domain_name_value",
                    location="location_value",
                )

                # Make the request
                response = await client.retrieve_register_parameters(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.RetrieveRegisterParametersRequest, dict]]):
                The request object. Request for the ``RetrieveRegisterParameters`` method.
            location (:class:`str`):
                Required. The location. Must be in the format
                ``projects/*/locations/*``.

                This corresponds to the ``location`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            domain_name (:class:`str`):
                Required. The domain name. Unicode
                domain names must be expressed in
                Punycode format.

                This corresponds to the ``domain_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.RetrieveRegisterParametersResponse:
                Response for the RetrieveRegisterParameters method.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [location, domain_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.RetrieveRegisterParametersRequest):
            request = domains.RetrieveRegisterParametersRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if location is not None:
            request.location = location
        if domain_name is not None:
            request.domain_name = domain_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.retrieve_register_parameters
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("location", request.location),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def register_domain(
        self,
        request: Optional[Union[domains.RegisterDomainRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        registration: Optional[domains.Registration] = None,
        yearly_price: Optional[money_pb2.Money] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Registers a new domain name and creates a corresponding
        ``Registration`` resource.

        Call ``RetrieveRegisterParameters`` first to check availability
        of the domain name and determine parameters like price that are
        needed to build a call to this method.

        A successful call creates a ``Registration`` resource in state
        ``REGISTRATION_PENDING``, which resolves to ``ACTIVE`` within
        1-2 minutes, indicating that the domain was successfully
        registered. If the resource ends up in state
        ``REGISTRATION_FAILED``, it indicates that the domain was not
        registered successfully, and you can safely delete the resource
        and retry registration.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_register_domain():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                registration = domains_v1.Registration()
                registration.domain_name = "domain_name_value"
                registration.contact_settings.privacy = "REDACTED_CONTACT_DATA"
                registration.contact_settings.registrant_contact.email = "email_value"
                registration.contact_settings.registrant_contact.phone_number = "phone_number_value"
                registration.contact_settings.admin_contact.email = "email_value"
                registration.contact_settings.admin_contact.phone_number = "phone_number_value"
                registration.contact_settings.technical_contact.email = "email_value"
                registration.contact_settings.technical_contact.phone_number = "phone_number_value"

                request = domains_v1.RegisterDomainRequest(
                    parent="parent_value",
                    registration=registration,
                )

                # Make the request
                operation = client.register_domain(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.RegisterDomainRequest, dict]]):
                The request object. Request for the ``RegisterDomain`` method.
            parent (:class:`str`):
                Required. The parent resource of the ``Registration``.
                Must be in the format ``projects/*/locations/*``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            registration (:class:`google.cloud.domains_v1.types.Registration`):
                Required. The complete ``Registration`` resource to be
                created.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            yearly_price (:class:`google.type.money_pb2.Money`):
                Required. Yearly price to register or
                renew the domain. The value that should
                be put here can be obtained from
                RetrieveRegisterParameters or
                SearchDomains calls.

                This corresponds to the ``yearly_price`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent, registration, yearly_price]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.RegisterDomainRequest):
            request = domains.RegisterDomainRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if registration is not None:
            request.registration = registration
        if yearly_price is not None:
            request.yearly_price = yearly_price

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.register_domain
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def retrieve_transfer_parameters(
        self,
        request: Optional[
            Union[domains.RetrieveTransferParametersRequest, dict]
        ] = None,
        *,
        location: Optional[str] = None,
        domain_name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.RetrieveTransferParametersResponse:
        r"""Gets parameters needed to transfer a domain name from another
        registrar to Cloud Domains. For domains managed by Google
        Domains, transferring to Cloud Domains is not supported.

        Use the returned values to call ``TransferDomain``.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_retrieve_transfer_parameters():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.RetrieveTransferParametersRequest(
                    domain_name="domain_name_value",
                    location="location_value",
                )

                # Make the request
                response = await client.retrieve_transfer_parameters(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.RetrieveTransferParametersRequest, dict]]):
                The request object. Request for the ``RetrieveTransferParameters`` method.
            location (:class:`str`):
                Required. The location. Must be in the format
                ``projects/*/locations/*``.

                This corresponds to the ``location`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            domain_name (:class:`str`):
                Required. The domain name. Unicode
                domain names must be expressed in
                Punycode format.

                This corresponds to the ``domain_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.RetrieveTransferParametersResponse:
                Response for the RetrieveTransferParameters method.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [location, domain_name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.RetrieveTransferParametersRequest):
            request = domains.RetrieveTransferParametersRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if location is not None:
            request.location = location
        if domain_name is not None:
            request.domain_name = domain_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.retrieve_transfer_parameters
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("location", request.location),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def transfer_domain(
        self,
        request: Optional[Union[domains.TransferDomainRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        registration: Optional[domains.Registration] = None,
        yearly_price: Optional[money_pb2.Money] = None,
        authorization_code: Optional[domains.AuthorizationCode] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Transfers a domain name from another registrar to Cloud Domains.
        For domains managed by Google Domains, transferring to Cloud
        Domains is not supported.

        Before calling this method, go to the domain's current registrar
        to unlock the domain for transfer and retrieve the domain's
        transfer authorization code. Then call
        ``RetrieveTransferParameters`` to confirm that the domain is
        unlocked and to get values needed to build a call to this
        method.

        A successful call creates a ``Registration`` resource in state
        ``TRANSFER_PENDING``. It can take several days to complete the
        transfer process. The registrant can often speed up this process
        by approving the transfer through the current registrar, either
        by clicking a link in an email from the registrar or by visiting
        the registrar's website.

        A few minutes after transfer approval, the resource transitions
        to state ``ACTIVE``, indicating that the transfer was
        successful. If the transfer is rejected or the request expires
        without being approved, the resource can end up in state
        ``TRANSFER_FAILED``. If transfer fails, you can safely delete
        the resource and retry the transfer.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_transfer_domain():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                registration = domains_v1.Registration()
                registration.domain_name = "domain_name_value"
                registration.contact_settings.privacy = "REDACTED_CONTACT_DATA"
                registration.contact_settings.registrant_contact.email = "email_value"
                registration.contact_settings.registrant_contact.phone_number = "phone_number_value"
                registration.contact_settings.admin_contact.email = "email_value"
                registration.contact_settings.admin_contact.phone_number = "phone_number_value"
                registration.contact_settings.technical_contact.email = "email_value"
                registration.contact_settings.technical_contact.phone_number = "phone_number_value"

                request = domains_v1.TransferDomainRequest(
                    parent="parent_value",
                    registration=registration,
                )

                # Make the request
                operation = client.transfer_domain(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.TransferDomainRequest, dict]]):
                The request object. Request for the ``TransferDomain`` method.
            parent (:class:`str`):
                Required. The parent resource of the ``Registration``.
                Must be in the format ``projects/*/locations/*``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            registration (:class:`google.cloud.domains_v1.types.Registration`):
                Required. The complete ``Registration`` resource to be
                created.

                You can leave ``registration.dns_settings`` unset to
                import the domain's current DNS configuration from its
                current registrar. Use this option only if you are sure
                that the domain's current DNS service does not cease
                upon transfer, as is often the case for DNS services
                provided for free by the registrar.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            yearly_price (:class:`google.type.money_pb2.Money`):
                Required. Acknowledgement of the price to transfer or
                renew the domain for one year. Call
                ``RetrieveTransferParameters`` to obtain the price,
                which you must acknowledge.

                This corresponds to the ``yearly_price`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            authorization_code (:class:`google.cloud.domains_v1.types.AuthorizationCode`):
                The domain's transfer authorization
                code. You can obtain this from the
                domain's current registrar.

                This corresponds to the ``authorization_code`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent, registration, yearly_price, authorization_code]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.TransferDomainRequest):
            request = domains.TransferDomainRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if registration is not None:
            request.registration = registration
        if yearly_price is not None:
            request.yearly_price = yearly_price
        if authorization_code is not None:
            request.authorization_code = authorization_code

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.transfer_domain
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def list_registrations(
        self,
        request: Optional[Union[domains.ListRegistrationsRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListRegistrationsAsyncPager:
        r"""Lists the ``Registration`` resources in a project.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_list_registrations():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ListRegistrationsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_registrations(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ListRegistrationsRequest, dict]]):
                The request object. Request for the ``ListRegistrations`` method.
            parent (:class:`str`):
                Required. The project and location from which to list
                ``Registration``\ s, specified in the format
                ``projects/*/locations/*``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.services.domains.pagers.ListRegistrationsAsyncPager:
                Response for the ListRegistrations method.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ListRegistrationsRequest):
            request = domains.ListRegistrationsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_registrations
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListRegistrationsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_registration(
        self,
        request: Optional[Union[domains.GetRegistrationRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.Registration:
        r"""Gets the details of a ``Registration`` resource.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_get_registration():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.GetRegistrationRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_registration(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.GetRegistrationRequest, dict]]):
                The request object. Request for the ``GetRegistration`` method.
            name (:class:`str`):
                Required. The name of the ``Registration`` to get, in
                the format ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.Registration:
                The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.GetRegistrationRequest):
            request = domains.GetRegistrationRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_registration
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_registration(
        self,
        request: Optional[Union[domains.UpdateRegistrationRequest, dict]] = None,
        *,
        registration: Optional[domains.Registration] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates select fields of a ``Registration`` resource, notably
        ``labels``. To update other fields, use the appropriate custom
        update method:

        - To update management settings, see
          ``ConfigureManagementSettings``
        - To update DNS configuration, see ``ConfigureDnsSettings``
        - To update contact information, see
          ``ConfigureContactSettings``

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_update_registration():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.UpdateRegistrationRequest(
                )

                # Make the request
                operation = client.update_registration(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.UpdateRegistrationRequest, dict]]):
                The request object. Request for the ``UpdateRegistration`` method.
            registration (:class:`google.cloud.domains_v1.types.Registration`):
                Fields of the ``Registration`` to update.
                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Required. The field mask describing which fields to
                update as a comma-separated list. For example, if only
                the labels are being updated, the ``update_mask`` is
                ``"labels"``.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration, update_mask]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.UpdateRegistrationRequest):
            request = domains.UpdateRegistrationRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.update_registration
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration.name", request.registration.name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def configure_management_settings(
        self,
        request: Optional[
            Union[domains.ConfigureManagementSettingsRequest, dict]
        ] = None,
        *,
        registration: Optional[str] = None,
        management_settings: Optional[domains.ManagementSettings] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates a ``Registration``'s management settings.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_configure_management_settings():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ConfigureManagementSettingsRequest(
                    registration="registration_value",
                )

                # Make the request
                operation = client.configure_management_settings(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ConfigureManagementSettingsRequest, dict]]):
                The request object. Request for the ``ConfigureManagementSettings`` method.
            registration (:class:`str`):
                Required. The name of the ``Registration`` whose
                management settings are being updated, in the format
                ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            management_settings (:class:`google.cloud.domains_v1.types.ManagementSettings`):
                Fields of the ``ManagementSettings`` to update.
                This corresponds to the ``management_settings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Required. The field mask describing which fields to
                update as a comma-separated list. For example, if only
                the transfer lock is being updated, the ``update_mask``
                is ``"transfer_lock_state"``.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration, management_settings, update_mask]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ConfigureManagementSettingsRequest):
            request = domains.ConfigureManagementSettingsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration
        if management_settings is not None:
            request.management_settings = management_settings
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.configure_management_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration", request.registration),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def configure_dns_settings(
        self,
        request: Optional[Union[domains.ConfigureDnsSettingsRequest, dict]] = None,
        *,
        registration: Optional[str] = None,
        dns_settings: Optional[domains.DnsSettings] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates a ``Registration``'s DNS settings.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_configure_dns_settings():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ConfigureDnsSettingsRequest(
                    registration="registration_value",
                )

                # Make the request
                operation = client.configure_dns_settings(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ConfigureDnsSettingsRequest, dict]]):
                The request object. Request for the ``ConfigureDnsSettings`` method.
            registration (:class:`str`):
                Required. The name of the ``Registration`` whose DNS
                settings are being updated, in the format
                ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            dns_settings (:class:`google.cloud.domains_v1.types.DnsSettings`):
                Fields of the ``DnsSettings`` to update.
                This corresponds to the ``dns_settings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Required. The field mask describing which fields to
                update as a comma-separated list. For example, if only
                the name servers are being updated for an existing
                Custom DNS configuration, the ``update_mask`` is
                ``"custom_dns.name_servers"``.

                When changing the DNS provider from one type to another,
                pass the new provider's field name as part of the field
                mask. For example, when changing from a Google Domains
                DNS configuration to a Custom DNS configuration, the
                ``update_mask`` is ``"custom_dns"``. //

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration, dns_settings, update_mask]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ConfigureDnsSettingsRequest):
            request = domains.ConfigureDnsSettingsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration
        if dns_settings is not None:
            request.dns_settings = dns_settings
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.configure_dns_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration", request.registration),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def configure_contact_settings(
        self,
        request: Optional[Union[domains.ConfigureContactSettingsRequest, dict]] = None,
        *,
        registration: Optional[str] = None,
        contact_settings: Optional[domains.ContactSettings] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates a ``Registration``'s contact settings. Some changes
        require confirmation by the domain's registrant contact .

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_configure_contact_settings():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ConfigureContactSettingsRequest(
                    registration="registration_value",
                )

                # Make the request
                operation = client.configure_contact_settings(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ConfigureContactSettingsRequest, dict]]):
                The request object. Request for the ``ConfigureContactSettings`` method.
            registration (:class:`str`):
                Required. The name of the ``Registration`` whose contact
                settings are being updated, in the format
                ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            contact_settings (:class:`google.cloud.domains_v1.types.ContactSettings`):
                Fields of the ``ContactSettings`` to update.
                This corresponds to the ``contact_settings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Required. The field mask describing which fields to
                update as a comma-separated list. For example, if only
                the registrant contact is being updated, the
                ``update_mask`` is ``"registrant_contact"``.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration, contact_settings, update_mask]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ConfigureContactSettingsRequest):
            request = domains.ConfigureContactSettingsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration
        if contact_settings is not None:
            request.contact_settings = contact_settings
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.configure_contact_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration", request.registration),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def export_registration(
        self,
        request: Optional[Union[domains.ExportRegistrationRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Exports a ``Registration`` resource, such that it is no longer
        managed by Cloud Domains.

        When an active domain is successfully exported, you can continue
        to use the domain in `Google
        Domains <https://domains.google/>`__ until it expires. The
        calling user becomes the domain's sole owner in Google Domains,
        and permissions for the domain are subsequently managed there.
        The domain does not renew automatically unless the new owner
        sets up billing in Google Domains.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_export_registration():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ExportRegistrationRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.export_registration(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ExportRegistrationRequest, dict]]):
                The request object. Request for the ``ExportRegistration`` method.
            name (:class:`str`):
                Required. The name of the ``Registration`` to export, in
                the format ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.domains_v1.types.Registration` The Registration resource facilitates managing and configuring domain name
                   registrations.

                   There are several ways to create a new Registration
                   resource:

                   To create a new Registration resource, find a
                   suitable domain name by calling the SearchDomains
                   method with a query to see available domain name
                   options. After choosing a name, call
                   RetrieveRegisterParameters to ensure availability and
                   obtain information like pricing, which is needed to
                   build a call to RegisterDomain.

                   Another way to create a new Registration is to
                   transfer an existing domain from another registrar.
                   First, go to the current registrar to unlock the
                   domain for transfer and retrieve the domain's
                   transfer authorization code. Then call
                   RetrieveTransferParameters to confirm that the domain
                   is unlocked and to get values needed to build a call
                   to TransferDomain.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ExportRegistrationRequest):
            request = domains.ExportRegistrationRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.export_registration
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            domains.Registration,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def delete_registration(
        self,
        request: Optional[Union[domains.DeleteRegistrationRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Deletes a ``Registration`` resource.

        This method works on any ``Registration`` resource using
        `Subscription or Commitment
        billing </domains/pricing#billing-models>`__, provided that the
        resource was created at least 1 day in the past.

        For ``Registration`` resources using `Monthly
        billing </domains/pricing#billing-models>`__, this method works
        if:

        - ``state`` is ``EXPORTED`` with ``expire_time`` in the past
        - ``state`` is ``REGISTRATION_FAILED``
        - ``state`` is ``TRANSFER_FAILED``

        When an active registration is successfully deleted, you can
        continue to use the domain in `Google
        Domains <https://domains.google/>`__ until it expires. The
        calling user becomes the domain's sole owner in Google Domains,
        and permissions for the domain are subsequently managed there.
        The domain does not renew automatically unless the new owner
        sets up billing in Google Domains.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_delete_registration():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.DeleteRegistrationRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.delete_registration(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.DeleteRegistrationRequest, dict]]):
                The request object. Request for the ``DeleteRegistration`` method.
            name (:class:`str`):
                Required. The name of the ``Registration`` to delete, in
                the format ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.DeleteRegistrationRequest):
            request = domains.DeleteRegistrationRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.delete_registration
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=domains.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def retrieve_authorization_code(
        self,
        request: Optional[Union[domains.RetrieveAuthorizationCodeRequest, dict]] = None,
        *,
        registration: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.AuthorizationCode:
        r"""Gets the authorization code of the ``Registration`` for the
        purpose of transferring the domain to another registrar.

        You can call this method only after 60 days have elapsed since
        the initial domain registration.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_retrieve_authorization_code():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.RetrieveAuthorizationCodeRequest(
                    registration="registration_value",
                )

                # Make the request
                response = await client.retrieve_authorization_code(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.RetrieveAuthorizationCodeRequest, dict]]):
                The request object. Request for the ``RetrieveAuthorizationCode`` method.
            registration (:class:`str`):
                Required. The name of the ``Registration`` whose
                authorization code is being retrieved, in the format
                ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.AuthorizationCode:
                Defines an authorization code.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.RetrieveAuthorizationCodeRequest):
            request = domains.RetrieveAuthorizationCodeRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.retrieve_authorization_code
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration", request.registration),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def reset_authorization_code(
        self,
        request: Optional[Union[domains.ResetAuthorizationCodeRequest, dict]] = None,
        *,
        registration: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> domains.AuthorizationCode:
        r"""Resets the authorization code of the ``Registration`` to a new
        random string.

        You can call this method only after 60 days have elapsed since
        the initial domain registration.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import domains_v1

            async def sample_reset_authorization_code():
                # Create a client
                client = domains_v1.DomainsAsyncClient()

                # Initialize request argument(s)
                request = domains_v1.ResetAuthorizationCodeRequest(
                    registration="registration_value",
                )

                # Make the request
                response = await client.reset_authorization_code(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.domains_v1.types.ResetAuthorizationCodeRequest, dict]]):
                The request object. Request for the ``ResetAuthorizationCode`` method.
            registration (:class:`str`):
                Required. The name of the ``Registration`` whose
                authorization code is being reset, in the format
                ``projects/*/locations/*/registrations/*``.

                This corresponds to the ``registration`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.domains_v1.types.AuthorizationCode:
                Defines an authorization code.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [registration]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, domains.ResetAuthorizationCodeRequest):
            request = domains.ResetAuthorizationCodeRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if registration is not None:
            request.registration = registration

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.reset_authorization_code
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("registration", request.registration),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "DomainsAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("DomainsAsyncClient",)
